@@ -1,12 +1,12 @@
 quantreg.rfsrc <- function(formula, data, object, newdata,
-                              method = "forest", splitrule = NULL,
-                              prob = NULL, prob.epsilon = NULL,
-                              oob = TRUE, fast = FALSE, maxn = 1e3, ...)
+                           method = "local", splitrule = NULL,
+                           prob = NULL, prob.epsilon = NULL,
+                           oob = TRUE, fast = FALSE, maxn = 1e3, ...)
 {
   ## we intialize the grow primary operation as false
   grow <- FALSE
   ## which method will be used?
-  method <- match.arg(method, c("forest", "gk", "GK", "G-K", "g-k"))
+  method <- match.arg(method, c("forest", "local", "gk", "GK", "G-K", "g-k"))
   ## we now allow other regression splitting rules to be used 
   ## splitrule <- "quantile.regr"
   ## --------------------------------------------------------------------------------
@@ -19,20 +19,29 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
   if (missing(object)) {
     ## the primary operation is grow 
     grow <- TRUE
-    ## pull unnamed parameters to pass to rfsrc
+    ## list of forest parameters
+    rfnames <- get.rfnames(hidden = TRUE)
+     
+    ## restrict to allowed values
+    rfnames <- rfnames[rfnames != "formula"            &
+                       rfnames != "data"               &
+                       rfnames != "splitrule"          ]
+    ## get the permissible hidden options
     dots <- list(...)
-    dots$formula <- dots$data <- NULL
-    ## if prob and prob.epsilon are NULL, leave them alone
+    dots <- dots[names(dots) %in% rfnames]
+    ## manually set key hidden options 
     dots$prob <- prob
     dots$prob.epsilon <- prob.epsilon
     ## set the default split rule
     if (is.null(splitrule)) {
       splitrule <- "la.quantile.regr"
     }
-    ## TBD TBD quantile splitrule fails in the case of multivariate regression
+    ## quantile splitrule cannot be used for multivariate regression
     fmly <- parseFormula(formula, data)$family
     if (fmly == "regr+" || fmly == "class+" || fmly == "mix+") {
-      splitrule <- NULL
+      if (splitrule == "la.quantile.regr" || splitrule == "la.quantile.regr") {
+        splitrule <- NULL
+      }
     }
     ## determine the grow interface - rfsrc or rfsrc.fast?
     if (!fast) {
@@ -46,6 +55,11 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
       dots$gk.quantile <- FALSE
       dots$forest.wt <- if (oob) "oob" else TRUE
     }
+    ## if user wants local method, set GK to false
+    else if (method == "local") {
+      dots$gk.quantile <- FALSE
+    }
+    ## user is requesting GK method
     else {
       dots$gk.quantile <- TRUE
     }
@@ -101,7 +115,7 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
   ##
   ## -----------------------------------------------------------------------
   if (!(object$family == "regr" | !is.null(object$regrOutput))) {
-    stop("this function only applies to regression settings\n")
+    stop("this function requires at least one outcome that is continuous\n")
   }
   ## ---------------------------------------
   ##
@@ -139,6 +153,11 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
         dots$forest.wt <- "oob"
       }
     }
+    ## local method -> set GK to false and request forest weights
+    else if (method == "local") {
+      dots$gk.quantile <- FALSE
+    }
+    ## user is requesting GK method
     else {
       dots$gk.quantile <- TRUE
     }
@@ -179,15 +198,13 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
   ## finally - extract quantile information
   ##
   ## -----------------------------------
-  ## step function interpolation
-  sIndex <- function(x1, x2) {sapply(1:length(x2), function(j) {sum(x1 <= x2[j])})}
   rO <- lapply(ynames, function(yn) {
     ## ------------------------------------
     ##
     ## GK method - also clean up the object   
     ##
     ## ------------------------------------
-    if (method != "forest") {
+    if (!(method == "forest" || method == "local")) {
       if (ncol(cbind(object$yvar.grow)) > 1) {
         y <- object$yvar.grow[, yn]
         if (oob && !is.null(object$regrOutput[[yn]]$quantile.oob)) {
@@ -224,29 +241,46 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
     }
     ## ------------------------------------
     ##
-    ## forest weight method
+    ## forest weight or local method
     ##
     ## ------------------------------------
     else {
-      ## pull grow y and grow residual
+      ## pull grow y, grow residual, and yhat (either grow/predicted)
       if (ncol(cbind(object$yvar.grow)) > 1) {
         y <- object$yvar.grow[, yn]
         res <- object$res.grow[, yn]
+        if (oob && !is.null(object$regrOutput[[yn]]$predicted.oob)) {
+          yhat <- object$regrOutput[[yn]]$predicted.oob
+        }
+        else {
+          yhat <- object$regrOutput[[yn]]$predicted
+        }
       }
       else {
         y <- object$yvar.grow
         res <- object$res.grow
+         if (oob && !is.null(object$predicted.oob)) {
+          yhat <- object$predicted.oob
+        }
+        else {
+          yhat <- object$predicted
+        }
       }
-      ## now obtain grow yhat 
-      yhat <- y - res
-      ## unique y values
+      ## extract unique y values
       yunq <- sort(unique(y))
-      ## locally adjusted cdf (now correctly applies to both grow and predict mode)
-      ## predict mode only needs predict forest weights, all other quantities are grow values 
-      cdf <- do.call(rbind, mclapply(1:nrow(object$forest.wt), function(i) {
-        ind.matx <- do.call(rbind, lapply(yunq, function(yy) {res <= (yy - yhat[i])}))
-        c(ind.matx %*% object$forest.wt[i, ])
-      }))
+      ## forest weight method, locally adjusts cdf using forest weights
+      if (method == "forest") {
+        cdf <- do.call(rbind, mclapply(1:nrow(object$forest.wt), function(i) {
+          ind.matx <- do.call(rbind, lapply(yunq, function(yy) {res <= (yy - yhat[i])}))
+          c(ind.matx %*% object$forest.wt[i, ])
+        }))
+      }
+      else {
+        ## locally adjusted cdf 
+        cdf <- do.call(rbind, mclapply(1:length(yhat), function(i) {
+          sapply(yunq, function(yy) {mean(res <= (yy - yhat[i]))})
+        }))
+      }
       ## quantiles
       quant <- t(apply(cdf, 1, function(pr) {
         c(min(yunq, na.rm = TRUE), yunq)[1 + sIndex(pr, prob)]
@@ -254,14 +288,23 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
       ## density
       density <- t(apply(cbind(0, cdf), 1, diff))
     }
-    ## return the object
+    ## ------------------------------------
+    ##
+    ## ends loop for specific yvar
+    ## return various quantities as list
+    ##
+    ## ------------------------------------
     list(quantiles = quant,
          prob = prob,
          cdf = cdf,
          density = density,
          yunq = yunq)
   })
-  ## return the goodies
+  ## ------------------------------------
+  ##
+  ## finished -- return final goodies
+  ##
+  ## ------------------------------------
   if (object$family == "regr") {
     rO <- rO[[1]]
   }
@@ -273,123 +316,3 @@ quantreg.rfsrc <- function(formula, data, object, newdata,
   object
 }
 quantreg <- quantreg.rfsrc
-##--------------------------------------------------------------
-##
-## wrappers for pulling desired quantitites
-##
-##--------------------------------------------------------------
-## pulls the quantile object and converts to a list
-extract.quantile <- function(o) {
-  ## confirm this is a quantile regression object
-  if (sum(grepl("quantreg", class(o))) == 0) { 
-    stop("object must be a quantreg object")
-  }
-  ## used to determine univariate vs multivariate objects
-  mv.y.names <- intersect(names(o$quantreg), o$yvar.names)
-  ## we have a univariate quantile regression object
-  if (length(mv.y.names) == 0) {
-    q <- list(o$quantreg)
-    names(q) <- o$yvar.names
-  }
-  ## the quantile regression object is multivariate
-  else {
-    q <- lapply(mv.y.names, function(m.target) {
-      o$quantreg[[m.target]]
-    })
-    names(q) <- mv.y.names
-  }
-  ## return the processed list
-  q
-}
-## extract target quantiles
-get.quantile <- function(o, target.prob = NULL, pretty = TRUE) {
-  ## extract the quantile object
-  qo <- extract.quantile(o)
-  ## process the target probabilities
-  if (!is.null(target.prob)) {
-    target.prob <- sort(unique(target.prob))
-  }
-  else {## default is to use existing values
-    target.prob <- qo[[1]]$prob
-  }
-  ## pull the target quantiles
-  rO <- lapply(qo, function(q) {    
-    q.dat <- do.call(cbind, lapply(target.prob, function(pr) {
-      q$quant[, which.min(abs(pr - q$prob))]
-    }))
-    colnames(q.dat) <-  paste("q.", 100 * target.prob, sep = "")
-    q.dat
-  })
-  if (pretty && length(rO) == 1) {
-    rO <- rO[[1]]
-  }
-  rO
-}
-## extract crps
-get.quantile.crps <- function(o, pretty = TRUE, subset = NULL) {
-  ## extract the quantile object
-  qO <- extract.quantile(o)
-  ## does not apply to predict objects without y
-  if (sum(grepl("predict", class(o))) > 0 && is.null(o$yvar)) {
-    stop("no yvar present in quantreg predict object")
-  }
-  ## subset assignment
-  if (is.null(subset)) {
-    subset <- 1:o$n
-  }
-  else {
-    if (is.logical(subset)) {
-      subset <- which(subset)
-    }
-    subset <- subset[subset >=1 & subset <= o$n]
-  }
-  if (length(subset) == 0) {
-    stop("requested subset is empty")
-  }
-  ## pull the target stats
-  rO <- lapply(1:length(qO), function(j) {
-    q <- qO[[j]]
-    if (is.vector(o$yvar)) {
-      y <- cbind(o$yvar)
-    }
-    else {
-      y <- o$yvar[, names(qO)[j]]
-    }
-    n <- length(y)
-    ## brier score
-    brS <- colMeans(do.call(rbind, mclapply(subset, function(i) {
-      (1 * (y[i] <= q$yunq) - q$cdf[i, ]) ^ 2 
-    })), na.rm = TRUE)
-    ## crps
-    crps <- unlist(lapply(1:length(q$yunq), function(j) {
-      trapz(q$yunq[1:j], brS[1:j]) / diff(range(q$yunq[1:j]))
-    }))
-    data.frame(y = q$yunq, crps = crps)
-  })
-  names(rO) <- names(qO)
-  if (pretty && length(rO) == 1) {
-    rO <- rO[[1]]
-  }
-  rO
-}
-## extract target stats
-get.quantile.stat <- function(o, pretty = TRUE) {
-  ## extract the quantile object
-  qO <- extract.quantile(o)
-  ## conditional median
-  mdn <- get.quantile(o, .5, FALSE)
-  ## pull the target stats
-  rO <- lapply(1:length(qO), function(j) {
-    q <- qO[[j]]
-    ## conditional mean
-    mn <- q$density %*% q$yunq
-    ## conditional standard deviation
-    std <- sqrt(q$density %*% q$yunq^2 - mn ^ 2)
-    data.frame(mean = mn, median = c(mdn[[j]]), std = std)
-  })
-  names(rO) <- names(qO)
-  if (pretty && length(rO) == 1) {
-    rO <- rO[[1]]
-  }
-  rO
-}
